"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.NextAuthHandler = NextAuthHandler;
exports.default = void 0;

var _logger = _interopRequireWildcard(require("../lib/logger"));

var routes = _interopRequireWildcard(require("./routes"));

var _pages = _interopRequireDefault(require("./pages"));

var _init = require("./init");

var _next = _interopRequireDefault(require("../next"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

async function NextAuthHandler(params) {
  var _req$body$callbackUrl, _req$cookies, _req$cookies2;

  const {
    userOptions,
    ...req
  } = params;

  if (userOptions.logger) {
    (0, _logger.setLogger)(userOptions.logger);
  }

  if (!process.env.NEXTAUTH_URL) {
    _logger.default.warn("NEXTAUTH_URL");
  }

  if (!req.query.nextauth) {
    const message = "Cannot find [...nextauth].js in pages/api/auth. Make sure the filename is written correctly.";

    _logger.default.error("MISSING_NEXTAUTH_API_ROUTE_ERROR", new Error(message));

    return {
      status: 500,
      text: `Error: ${message}`
    };
  }

  const {
    nextauth,
    action = nextauth[0],
    providerId = nextauth[1],
    error = nextauth[1]
  } = req.query;
  delete req.query.nextauth;
  const {
    options,
    cookies
  } = await (0, _init.init)({
    userOptions,
    action,
    providerId,
    callbackUrl: (_req$body$callbackUrl = req.body.callbackUrl) !== null && _req$body$callbackUrl !== void 0 ? _req$body$callbackUrl : req.query.callbackUrl,
    csrfToken: req.body.csrfToken,
    cookies: req.cookies,
    isPost: req.method === "POST"
  });
  const sessionToken = (_req$cookies = req.cookies) === null || _req$cookies === void 0 ? void 0 : _req$cookies[options.cookies.sessionToken.name];
  const codeVerifier = (_req$cookies2 = req.cookies) === null || _req$cookies2 === void 0 ? void 0 : _req$cookies2[options.cookies.pkceCodeVerifier.name];

  if (req.method === "GET") {
    const render = (0, _pages.default)({
      options,
      query: req.query,
      cookies
    });
    const {
      pages
    } = options;

    switch (action) {
      case "providers":
        return {
          json: routes.providers(options.providers),
          cookies
        };

      case "session":
        {
          const session = await routes.session({
            options,
            sessionToken
          });
          if (session.cookie) cookies.push(session.cookie);
          return {
            json: session.json,
            cookies
          };
        }

      case "csrf":
        return {
          json: {
            csrfToken: options.csrfToken
          },
          cookies
        };

      case "signin":
        if (pages.signIn) {
          let signinUrl = `${pages.signIn}${pages.signIn.includes("?") ? "&" : "?"}callbackUrl=${options.callbackUrl}`;

          if (error) {
            signinUrl = `${signinUrl}&error=${error}`;
          }

          return {
            redirect: signinUrl,
            cookies
          };
        }

        return render.signin();

      case "signout":
        if (pages.signOut) return {
          redirect: pages.signOut,
          cookies
        };
        return render.signout();

      case "callback":
        if (options.provider) {
          const callback = await routes.callback({
            body: req.body,
            query: req.query,
            method: req.method,
            headers: req.headers,
            options,
            sessionToken,
            codeVerifier
          });
          if (callback.cookies) cookies.push(...callback.cookies);
          return { ...callback,
            cookies
          };
        }

        break;

      case "verify-request":
        if (pages.verifyRequest) {
          return {
            redirect: pages.verifyRequest,
            cookies
          };
        }

        return render.verifyRequest();

      case "error":
        if (pages.error) {
          return {
            redirect: `${pages.error}${pages.error.includes("?") ? "&" : "?"}error=${error}`,
            cookies
          };
        }

        if (["Signin", "OAuthSignin", "OAuthCallback", "OAuthCreateAccount", "EmailCreateAccount", "Callback", "OAuthAccountNotLinked", "EmailSignin", "CredentialsSignin", "SessionRequired"].includes(error)) {
          return {
            redirect: `${options.base}/signin?error=${error}`,
            cookies
          };
        }

        return render.error({
          error
        });

      default:
    }
  } else if (req.method === "POST") {
    switch (action) {
      case "signin":
        if (options.csrfTokenVerified && options.provider) {
          const signin = await routes.signin({
            query: req.query,
            body: req.body,
            options
          });

          if (signin.cookies) {
            cookies.push(...signin.cookies);
          }

          return { ...signin,
            cookies
          };
        }

        return {
          redirect: `${options.base}/signin?csrf=true`,
          cookies
        };

      case "signout":
        if (options.csrfTokenVerified) {
          const signout = await routes.signout({
            options,
            sessionToken
          });
          if (signout.cookie) cookies.push(signout.cookie);
          return { ...signout,
            cookies
          };
        }

        return {
          redirect: `${options.base}/signout?csrf=true`,
          cookies
        };

      case "callback":
        if (options.provider) {
          if (options.provider.type === "credentials" && !options.csrfTokenVerified) {
            return {
              redirect: `${options.base}/signin?csrf=true`,
              cookies
            };
          }

          const callback = await routes.callback({
            body: req.body,
            query: req.query,
            method: req.method,
            headers: req.headers,
            options,
            sessionToken,
            codeVerifier
          });
          if (callback.cookies) cookies.push(...callback.cookies);
          return { ...callback,
            cookies
          };
        }

        break;

      case "_log":
        if (userOptions.logger) {
          try {
            const {
              code,
              level,
              ...metadata
            } = req.body;

            _logger.default[level](code, metadata);
          } catch (error) {
            _logger.default.error("LOGGER_ERROR", error);
          }
        }

        return {
          cookies
        };

      default:
    }
  }

  return {
    status: 400,
    cookies: []
  };
}

var _default = _next.default;
exports.default = _default;